#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === FindLineInFile
#
# The purpose of the class in this file is to find a specific line
# in a given file.
#
# If this file exists, then we will return a single integer
# which will denote the file number that contains our
# substring in question (starting at line number 1).
#
# Specific usage examples:
#
#   require 'find_line_in_file'
#   FindLineInFile.new
#   FindLineInFile.new('include Colours')
#   FindLineInFile.new('include Colours', 'att.rb')
#   FindLineInFile['run_everything', '/Users/x/DATA/PROGRAMMING_LANGUAGES/RUBY/src/tools/to_png.rb']
#
# Here is a slightly more complex usage example of FindLineInFile:
#
#   FindLineInFile[ what: 'run_everything', where: '/home/x/DATA/PROGRAMMING_LANGUAGES/RUBY/src/tools/to_png.rb']
#
# =========================================================================== #
# require 'find_line_in_file'; FindLineInFile.new
# =========================================================================== #
require 'colours'
require 'cliner'
require 'opn'
require 'find_line_in_file/class_methods.rb'
require 'find_line_in_file/constants.rb'
require 'find_line_in_file/version/version.rb'

class FindLineInFile # === FindLineInFile

  include Colours::E

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      search_for_this_line = nil,
      search_in_this_file  = nil,
      run_already          = true
    )
    reset
    # ======================================================================= #
    # The second one has to come first, because the first argument may
    # sometimes be a Hash. If it is a Hash then it may overrule the @file
    # variable, so that is why the order is reverse.
    # ======================================================================= #
    set_search_in_this_file(
      search_in_this_file  # This file is grepped.
    )
    set_search_for_this_line(
      search_for_this_line # This line is sought.
    )
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    # ======================================================================= #
    # === @data
    # ======================================================================= #
    @data   = nil
    # ======================================================================= #
    # === @result
    # ======================================================================= #
    @result = nil
    # ======================================================================= #
    # === @can_we_continue
    # ======================================================================= #
    @can_we_continue = true
    # ======================================================================= #
    # === @use_this_encoding
    # ======================================================================= #
    @use_this_encoding = USE_THIS_ENCODING
  end

  # ========================================================================= #
  # === search_for_which_file?
  # ========================================================================= #
  def search_for_which_file?
    @search_for_this_file
  end; alias file? search_for_which_file? # === file? 

  # ========================================================================= #
  # === clear_dataset
  # ========================================================================= #
  def clear_dataset # No more need for @data to contain anything.
    @data = nil; remove_instance_variable(:@data)
  end

  # ========================================================================= #
  # === can_we_continue?
  # ========================================================================= #
  def can_we_continue?
    @can_we_continue
  end

  # ========================================================================= #
  # === sfile
  # ========================================================================= #
  def sfile(i)
    ::Colours.sfile(i)
  end

  # ========================================================================= #
  # === ewarn
  # ========================================================================= #
  def ewarn(i)
    ::Colours.ewarn(i)
  end

  # ========================================================================= #
  # === simp
  # ========================================================================= #
  def simp(i)
    ::Colours.simp(i)
  end

  # ========================================================================= #
  # === swarn
  # ========================================================================= #
  def swarn(i)
    ::Colours.swarn(i)
  end

  # ========================================================================= #
  # === report_result
  #
  # We only report if we have found something.
  # ========================================================================= #
  def report_result
    if can_we_continue?
      opn; e "We will try to find something in "\
             "#{sfile(search_for_which_file?)} now."
      opn; e 'The variable @result (of class '+@result.class.to_s+') '+
             'is: '+simp(@result.to_s)
    end
  end; alias report report_result # === report

  # ========================================================================= #
  # === read_in_dataset
  #
  # This here makes use of File.readlines(), so the Encoding must be
  # checked.
  # ========================================================================= #
  def read_in_dataset
    @data = File.readlines(
      search_for_which_file?,
      encoding: @use_this_encoding # We must specify our default encoding to use.
    )
  end

  # ========================================================================= #
  # === number?
  # ========================================================================= #
  def number?
    @result
  end; alias line_number? number? # === line_number?

  # ========================================================================= #
  # === result?
  # ========================================================================= #
  def result?
    @result
  end; alias result result? # === result

  # ========================================================================= #
  # === set_search_in_this_file
  # ========================================================================= #
  def set_search_in_this_file(i = nil)
    i = DEFAULT_FILE if i.nil?
    i = i.first if i.is_a? Array
    @search_for_this_file = i
  end; alias set_this_file set_search_in_this_file # === set_this_file
       alias use_this_file set_search_in_this_file # === use_this_file

  # ========================================================================= #
  # === set_search_for_this_line
  # ========================================================================= #
  def set_search_for_this_line(i = nil)
    i = THIS_LINE if i.nil?
    if i.is_a? Hash
      if i.has_key? :this_file
        set_this_file(i.delete(:this_file))
      end
      if i.has_key? :this_line
        i = i.delete(:this_line)
      end
    end
    i = i.to_s
    @search_for_this_line = i
  end; alias search_for set_search_for_this_line # === search_for

  # ========================================================================= #
  # === check_whether_the_file_exists
  #
  # We find out whether our target file exists or whether it does not.
  # ========================================================================= #
  def check_whether_the_file_exists
    search_for_this_file = search_for_which_file?
    unless File.exist? search_for_this_file
      opn; ewarn 'The file `'+sfile(search_for_this_file)+
           swarn('` does not exist.')
      @can_we_continue = false
    end
  end

  # ========================================================================= #
  # === find_input_in_dataset
  # ========================================================================= #
  def find_input_in_dataset
    # ======================================================================= #
    # Must add +1 because files start at 1, whereas Arrays start at entry 0.
    # ======================================================================= #
    @result = @data.find_index {|entry|
      entry.include? @search_for_this_line.to_s
    }
    if @result
      @result += 1
    else
      opn; e "We may have not found anything for: `#{@search_for_this_line}`"
      pp @search_for_this_line
      opn; e 'Its encoding was: '+
             simp(@search_for_this_line.to_s.encoding.to_s)
    end
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    check_whether_the_file_exists
    if can_we_continue?
      read_in_dataset
      find_input_in_dataset
      clear_dataset
      return @result
    end
  end

end

if __FILE__ == $PROGRAM_NAME
  _ = FindLineInFile.new(ARGV.first, ARGV[1])
  _.report_result
end # find_line_in_file